// MagicLantern Widget
//   by Ben Sharpe
// Initially based on the Desktop Slideshow widget
//   by Karl Kranich - www.kranich.org

// Cycles through .bmp, .jpg, .tif,.png and .crw files in a folder, setting them as the desktop wallpaper
// optionally adds a nice slide frame to the images.
// use this code however you like

// 04/17/2007 - Version 2.0: Added UUID to widget.xml
//							 Added multiple frames
//							 Added fade transition (only works if widget is below others)
//                           Simplified Preferences a little
//							 New look and feel
//							 Added 'AND' tag search
//							 Used RunCommandInBg for intensive work
// 04/16/2007 - Version 1.5: Added flickr Tag Search
//							 Added Photo info bar
//							 Added show photographer page and show photo page to info bar
// 04/15/2007 - Version 1.4: Added buttons to Icon (Play/Pause, Next and Save Image)
// 04/13/2007 - Version 1.3: fixed bug with flickr badge
// 04/12/2007 - Version 1.2: added portrait preferences
// 04/11/2007 - Version 1.1: handles Portrait/Square images better
// 							 uses lanczos filter for better quality
// 03/24/2007 - Version 1.0: incorporate ImageMagick, add slideframe, add dock icon

// Lessons Learned:
//   runCommand likes quotes around paths with spaces in them
//   Windows functions (e.g. setDesktop) do not like quotes around paths.
//   You have to use extractFile() on anything used externally in a Widget Flat/Zip File
//   Math.random() appears to be *much* more random than the widget random() function
//   It seems you can't force a window to be on the very bottom

// TODO:
//   Show some exif info from local files
//   Download the next image async so that switching is faster
//   done 04/15/07 - Click the icon to save images you really like to your local folder
//   done 04/16/07 - Pick different images from Flickr (not just interestingness)
//   done 04/16/07 -  Show the title and photographer of the flickr images  briefly
//   done 04/15/07 - Have a plan for when flickr cannot be reached
// -------------------------------------------------------------------------------------------

// -------------------------------------------------------------------------------------------
// Globals -- what fun
// Main
var ImageList = new Array();
var imageCount = 0;
var pickedImage = null;
var origImage = null;
var aspectRatio = 1.0;
var imgWidth = 0;
var imgHeight = 0;
var deskWidth = screen.availWidth;
var deskHeight = screen.availHeight;
var flickrApiKey = "92ea9d2e94ef7b9fc3d5b1a80d211aa5";
var flickrPerPage = 10;
var flickrPages = [];
var flickrPage = 1;
var useFlickr = false;

var convert = quote(widget.extractFile("Helpers/convert.exe")) + " ";
var identify = quote(widget.extractFile("Helpers/identify.exe")) + " ";

var flickrFile = system.widgetDataFolder + "/flickrImage.jpg";
var wallpaper_file = system.temporaryFolder + '/__MagicLantern_Desktop__.bmp';
var dock_file = system.temporaryFolder + '/__MagicLantern_Dock__.png';
// -------------------------------------------------------------------------------------------
// indexOf is really handy
if (!Array.prototype.indexOf)
{
  Array.prototype.indexOf = function(elt /*, from*/)
  {
    var len = this.length;

    var from = Number(arguments[1]) || 0;
    from = (from < 0)
         ? Math.ceil(from)
         : Math.floor(from);
    if (from < 0)
      from += len;

    for (; from < len; from++)
    {
      if (from in this &&
          this[from] === elt)
        return from;
    }
    return -1;
  };
}
// -------------------------------------------------------------------------------------------
function quote(str) { return("\"" + (str) + "\" "); }

// -------------------------------------------------------------------------------------------
// This function pulled from the Yahoo Flickr Widget
function Arg( member, value )
{
	this.member = member ? member : "";
	this.value = value ? value : "";
}

// -------------------------------------------------------------------------------------------
function setWallpaper() {
    print("SetWallpaper()")
	showBlanket(true);
    showSaveButton(false);
    showInfoWindow(false);
    //Icon.src = "Resources/pOn.png";
    
	var size = deskWidth + "x" + deskHeight;
    
	orig_Image = pickImage();
    var cmdStr = "";


    var slideFramer = ""
    if (preferences.frameType.value != "None") {
        slideFramer = " ( " + quote(widget.extractFile("Resources/" + preferences.frameType.value)) 
        	+ " -scale \"" + size + "!\" )  -composite   "
    }
    
    var backgroundColor = preferences.bkgColor.value;
    
    // The Command
    var cmdStr = " \"" + orig_Image + "\" -resize \"" + size + "!\" -filter lanczos ";
    if (aspectRatio < 1.1) {
    	// Find an average color to use as the background on an image that doesn't fill the screen
    	if (preferences.autocolor.value == '1') {
	    	cmdStr = " \"" + orig_Image + "\" -gravity center -scale 1x1! -depth 8 txt:-";
	    	var tmp = (runCommand(convert + cmdStr)).split(' ');
	    	backgroundColor = tmp[tmp.length - 1];
		}
		if (backgroundColor == "#000000") {
			// Skip the slideframe if the background is black (because you can't see it)
			slideFramer = ""
		}
		
    	print("background: " + backgroundColor);
    	
    	if (preferences.stretch.value == '1') {
	    	cmdStr = " (  \"" + orig_Image + "\"  -resize \"" + size + "!\" -filter lanczos ) ";    		
    	}
    	else
    	{
	    	cmdStr = " (  \"" + orig_Image + "\"  -resize \"" + size + "\" -filter lanczos ) ";
	    	if (preferences.doBlur.value == "1") {
	    		cmdStr += " -matte -virtual-pixel transparent -channel A " + 
	    			"-blur 0x4 -evaluate subtract 50%  -evaluate multiply 2.001 ";
			}
		}
    	cmdStr += quote(wallpaper_file)
    	print(runCommand(convert + cmdStr));
   	
    	cmdStr = " -size " + size + " " + quote(wallpaper_file) + "  -background " + backgroundColor + 
    		" -gravity center -extent " + size + " ";
    }
    
    // The slide frame 
    cmdStr += slideFramer;
    
    // Fit to windwows screen size
    if ( screen.availHeight < screen.height ) {
    	var diff = screen.height - screen.availHeight;
    	print("Adding " + diff + " pixels to the bottom.");
    	cmdStr +=
    		" -background " + backgroundColor  + 
    		" -gravity north -extent " + screen.width + "x" + screen.height;
	}
    
    // The output file (note: bmp3 is required for windows to display this correctly)
    cmdStr += " -density " + screen.resolution + "x" + screen.resolution + "  bmp3:" + quote(wallpaper_file);
        
    // Print for debugging
    //print(cmdStr);
    runCommandInBg(convert + cmdStr, "setwallpaper");
        	
}

function finishSetWallpaper() {
	print("finishSetWallpaper()");
    applyWallpaper( wallpaper_file );
    updateInfo();
	canSaveImage();
	showBlanket(false);
    updateDockImage(orig_Image);
    setupTimer();	
}


// -------------------------------------------------------------------------------------------
function showNextImage() {
	// Save the desktop size, because it can change when a screensaver takes over
	deskWidth = screen.availWidth;
	deskHeight = screen.availHeight;
	setWallpaper();
}

// -------------------------------------------------------------------------------------------
function updateDockImage(orig_image) {
	print("updateDockImage()");
	
    var cmdStr = "";

    // The Command
    var cmdStr = quote(orig_image);
    
	// The Magick in ImageMagick
    if (aspectRatio >= 1.0) {
        cmdStr += " -scale x140 -scale \"150x<\" ";
    } else {
    	cmdStr += " -scale 150x -scale \"x140<\" ";
    }   
    cmdStr += " -scale 50% -background black -gravity center -crop 75x70+0+0 +repage "    + 
		quote(dock_file) ;

    // Print for debugging
    //print(convert + cmdStr);
    runCommandInBg(convert + cmdStr,"dockImage");
}

function finishDockImage() {
	    // Update the dock image
    var doc = XMLDOM.createDocument();
	var v = doc.createElement("dock-item");
	v.setAttribute("version","1.1");
	doc.appendChild(v);
	var image = doc.createElement("image");
	image.setAttribute("src", dock_file);   
	v.appendChild(image);  
	var badge = doc.createElement("image");
	if (useFlickr) {
		badge.setAttribute("src", "Resources/flickrBadge.png");
	} else {
		badge.setAttribute("src", "Resources/localBadge.png");
	}
	badge.setAttribute("hoffset", 0);
	badge.setAttribute("voffset", 50);
	v.appendChild(badge);
	
	widget.setDockItem( doc, "push-left");	
}


// -------------------------------------------------------------------------------------------
// based on code by David Zentgraf
function applyWallpaper(path_to_image) {
	if (system.platform == "macintosh") {
		appleScript('tell application \"Finder\" 			\
		to set desktop picture to \"' + convertPathToHFS(path_to_image) + '\"');
	} else if (system.platform == "windows") {
		changeWallpaperWindows(path_to_image);
	}
}

// -------------------------------------------------------------------------------------------
// based on code by Brian Fernandes
function changeWallpaperWindows(pathToBMPfile, tile, stretch){
    try { var wshShell = COM.createObject("WScript.Shell"); }
    catch(e) { beep(); alert("Unable to set wallpaper: " + e );}
    var regKey = "HKEY_CURRENT_USER\\Control Panel\\Desktop\\";
    wshShell.RegWrite(regKey + "Wallpaper", pathToBMPfile, "REG_SZ");
    var tMode = "0"; if (tile)    { tMode = "1"; }
    var sMode = "0"; if (stretch) { sMode = "2"; }
    wshShell.RegWrite(regKey + "TileWallpaper",  tMode, "REG_SZ");
    wshShell.RegWrite(regKey + "WallpaperStyle", sMode, "REG_SZ");
    runCommand("rundll32 user32.dll, UpdatePerUserSystemParameters");
    wshShell = null;
}

// -------------------------------------------------------------------------------------------
function checkDimensions( imgFile ) {
	var imageOK = false;
	
	if (useFlickr) {
		imgWidth = parseInt(pickedImage.width);
		imgHeight = parseInt(pickedImage.height);
	} else {
	 	var tmp = identify + " -format \"%w %h\" " + quote(imgFile);
	    //print(tmp);
	    tmp = runCommand(tmp);
	    //print(tmp);
	    var coords = tmp.split(" ");
	    imgWidth = coords[0];
	    imgHeight = coords[1];
	}
	
    if ((imgWidth < (deskWidth/3)) || (imgHeight < (deskHeight/3))) {
  	  	print("Image is too dang small @ " + imgWidth + "x" + imgHeight );
        imageOK = false;
    }
    else
    {
    	aspectRatio = imgWidth / imgHeight; 
    	print("aspectRatio = " + aspectRatio);
    	print("desktopRatio = " + (deskWidth / deskHeight));
        imageOK = !(aspectRatio < 1.1) ;
        if (preferences.portraitToo.value == "1") imageOK = true;
        if (aspectRatio > 2.0) {
        	imageOK = false;
        	print("Image too wide.");
        }
    }
	return imageOK;
}

// -------------------------------------------------------------------------------------------
function pickImage() {
    var numImages = ImageList.length;
    print("pickImage(" + numImages + " images)");
    var imageFolder = preferences.imageFolder.value + "/";
    

    // If there is only 1 image...that's the only one we can show
    if (ImageList.length == 0) return(('Resources/nofound.png'));
    
    var result = '';
    var imageOK = true;
    var curPick = 0;
 
    do {
        if ((numImages > 1) && (preferences.imageRandom.value == "1")) {
             // note: Math.random() is *much* more random than the widget random()
            curPick = parseInt(Math.random() * numImages);
            print("Random Pick: " + curPick + " of " + numImages);
        } 

		if (useFlickr) {
			pickedImage = ImageList[curPick];
			result = null;
			if (downloadFlickrImage(pickedImage.id)) {
				result = flickrFile;
			}
			print("Picking '" + pickedImage.title + "'");
		} else {
       		result = imageFolder + ImageList[curPick];
        	print("Picking '" + ImageList[curPick] + "'");
   		}
        ImageList.splice(curPick,1);
        numImages = ImageList.length;
        imageOK = false;
        if (result != null) {
        	imageOK = checkDimensions(result);
        }
        
    } while ((!imageOK) && (numImages > 0))
    
    if (numImages == 0) {
        collectImages();
        if (ImageList.length == 0) result = widget.extractfile('Resources/nofound.png');
    }
	
    return(result);
}

// -------------------------------------------------------------------------------------------
function flickrFilePath() {
	var destFoldr = preferences.flickrSaveFolder.value;
	if (destFoldr.length == 0) {
		destFoldr = preferences.imageFolder.value;
	}
	destFoldr += "/Flickr_" + pickedImage.id + ".jpg";	
	return(destFoldr);
}


// -------------------------------------------------------------------------------------------
function canSaveImage() {
	if (useFlickr) {
		showSaveButton(!filesystem.itemExists(flickrFilePath()));
	} else {
		showSaveButton(false);
	}
}


// -------------------------------------------------------------------------------------------
function saveImage() {
	if (pickedImage) {
		print("Saving: " + flickrFilePath());
		filesystem.copy(flickrFile, flickrFilePath());
	}
	showSaveButton(false);
}

// -------------------------------------------------------------------------------------------
function showBlanket( showit ) {
	print("showBlanket("+showit+")");
	if (showit && filesystem.itemExists(wallpaper_file) && (preferences.imageFade.value == '1')) {
		blanket.width = screen.width;
		blanket.height = screen.height;
//		print("screen " + screen.availWidth + "x" + screen.availHeight );
		print("blanket " + blanket.width + "x" + blanket.height );
		blanket_image.src = wallpaper_file;
		blanket_image.reload();
		blanket.opacity = 0;
		animator.runUntilDone(new FadeAnimation( blanket, 255, 500, animator.kEaseOut ) );
	} else {
		if (blanket.opacity > 0) {
			animator.runUntilDone(new FadeAnimation( blanket, 0, 500, animator.kEaseOut ) );
		}
		// Now use a smaller image to free up memory
		blanket_image.src = "Resources/unknownhead.png";
	}
}

// -------------------------------------------------------------------------------------------
function showSaveButton( showit ) {
	if (showit) {
		print("Show SaveButton");
		var a = new FadeAnimation( save, 255, 350, animator.kEaseOut );
		var b = new FadeAnimation( save_shadow, 255, 350, animator.kEaseOut );
		animator.start(new Array(a,b) );
	} else {
		print("Hide SaveButton");
		var a = new FadeAnimation( save, 0, 350, animator.kEaseOut );
		var b = new FadeAnimation( save_shadow, 0, 350, animator.kEaseOut );
		animator.start(new Array(a,b) );
	}
}

// -------------------------------------------------------------------------------------------
function showInfoWindow( showit ) {
	if (useFlickr) {
		if (showit && (preferences.flickrShowInfo.value == '1')) {
			animator.runUntilDone(new FadeAnimation( artist_info, 255, 350, animator.kEaseOut ) );
		} else {
			animator.runUntilDone(new FadeAnimation( artist_info, 0, 350, animator.kEaseOut ) );
		}
	}
}

// -------------------------------------------------------------------------------------------
function updateInfo() {
	if (useFlickr) {
		print(pickedImage.title);
		photographer_name.text = pickedImage.ownername;
		//var d = Date.(pickedImage.datetaken.split(' ')[0]);
		// Date.parse(pickedImage.datetaken);
		
		date_taken.text = pickedImage.datetaken.split(' ')[0];
		photo_name.text = pickedImage.title;
		if (pickedImage.title.length == 0) {
			photo_name.text = "untitled";
		}
		if( pickedImage.iconServer != 0 )
		{
			head.src = "http://static.flickr.com/" + pickedImage.iconserver + "/buddyicons/" + pickedImage.owner + ".jpg";
			//print(head.src);
		} else {
			head.src = "Resources/unknownHead.png";
		}
	}
	showInfoWindow(useFlickr);
}


// -------------------------------------------------------------------------------------------
function collectImages() {
	useFlickr = !(preferences.imgSource.value == 'Local Folder');
	showSaveButton(useFlickr);
	if (useFlickr) {
		print("Using Flickr");
		if (!collectFlickrImages()) {
			useFlickr = false;
			collectLocalImages();
		}
	} else {
		print("Using Local Folder");
		collectLocalImages();
	}
	Icon.tooltip = imageCount + " images found"
}

// -------------------------------------------------------------------------------------------
function collectLocalImages() {
    print("collectLocalImages()");
    
    var imageFolder = preferences.imageFolder.value + "/";
    var doSubfolders = (preferences.folderRecurse.value == "1");
    ImageList = filesystem.getDirectoryContents( imageFolder, doSubfolders );
    
    // Remove anything that's not a valid image from the list
    var remove = false;
    var removeCount = 0;
    var originalCount = ImageList.length;
    for (var i = ImageList.length - 1; i >= 0; i --) {
     	// The following tests are based on code from the Yahoo "Picture Frame" widget
	   	img = ImageList[i]
        remove = (filesystem.isDirectory(img)) ||
        		 (img.indexOf('/.') > -1) ||
        		 (img.indexOf('Thumbs') > -1) ||
        		 (img.indexOf('Trashbox') > -1 ||
        		 (!img.match(/.*(bmp|jpg|jpeg|gif|png|tif|tiff|crw)$/i)));
        if (remove) {
            ImageList.splice(i,1);
            removeCount++;
        }
    }

    print(originalCount + " found - " + removeCount + " removed = " + ImageList.length);
    imageCount = ImageList.length;
    source.tooltip = 'Local Images: ' + imageCount;
    source.src = "Resources/localbadge.png";
    source.vOffset = 8;
    source.hOffset = 12;
    source.reload();
}

// -------------------------------------------------------------------------------------------
function collectFlickrImages() {
	print("collectFlickrImages("+preferences.flickrSearch.value+": "+preferences.flickrOption.value+")");
	if (flickrPages.length == 0) {
		flickrPages = [1];
	}
	
	var index = 0;
	if (preferences.imageRandom.value == "1") {
		if (flickrPages.length > 1) {
			index = parseInt(Math.random() * flickrPages.length); 
		} 
	} 
	flickrPage = flickrPages[index];
	flickrPages.splice(index,1);
	print("  From Page: " + flickrPage);
	
	if (preferences.flickrSearch.value == 'Interestingness') {
		result = callFlickrMethod( "flickr.interestingness.getList", 
			[new Arg( "page", flickrPage ), 
			 new Arg( "per_page", flickrPerPage ),
			 new Arg( "extras", "owner_name,icon_server,date_taken" )] );
	} else if (preferences.flickrSearch.value == 'Tag Search') {
		var args = [new Arg( "page", flickrPage ), 
			 new Arg( "per_page", flickrPerPage ),
			 new Arg( "tags", preferences.flickrOption.value ),
			 new Arg( "extras", "owner_name,icon_server,date_taken" )];
		if (preferences.flickrAllTags.value == '1') {
			args = args.concat( new Arg( "tag_mode", "all" ) );
		}
		result = callFlickrMethod( "flickr.photos.search", args );
	}

	
//	result = callFlickrMethod( "flickr.interestingness.getList", 

	if (result.stat != "ok") {
		// bork
		print("Error: " + result.message);
		return false;
	} else {
		print("Found: " + result.photos.total + " images, Page: " + flickrPage + " of " + result.photos.pages + " @ " + flickrPerPage + " per page");
		imageCount = result.photos.total;
	}
	
	if (flickrPages.length == 0) {
		for (var i = 0; i < result.photos.pages; i++) {
			flickrPages = flickrPages.concat(i);
		}
	}
	
	ImageList = result.photos.photo;
    source.tooltip = 'Flickr Images: ' + imageCount;
    source.src = "Resources/flickrbadge.png";
    source.vOffset = 12;
    source.hOffset = 10;
    source.reload();
	
	return true;
}

// -------------------------------------------------------------------------------------------
function downloadFlickrImage( id ) {
	result = callFlickrMethod( "flickr.photos.getSizes", 
			[new Arg( "photo_id", id )] );
	if (result.stat != "ok") {
		return false;
	}
	
	// Find the largest version of the image available and download it
	sizes = result.sizes.size;
	img_Url = "";
	var pickSizes = ['Medium','Large','Original'];
	var pickIndex = -1;
	for( var i=0; i < sizes.length; i++) {
		var idx = pickSizes.indexOf(sizes[i].label);
		if (idx > pickIndex) {
			pickIndex = idx;
			img_Url = sizes[i].source;
			pickedImage.width = sizes[i].width;
			pickedImage.height = sizes[i].height;
		}
	}
	print("Using the " + pickSizes[pickIndex] + " Image");
	
	var url = new URL();
	url.outputFile = flickrFile;
	url.location = img_Url;
	url.fetch();

	return true;
}

// -------------------------------------------------------------------------------------------
function jsonFlickrApi( result ) { return(result); }

// -------------------------------------------------------------------------------------------
// This function pulled from the Yahoo Flickr Widget and modified to do less work
function callFlickrMethod( methodName, argList )
{
	if( !argList )
	{
		argList = new Array( );
	}

	argList.push( new Arg( "api_key", flickrApiKey ) );
	argList.push( new Arg( "method", methodName ) );
	argList.push( new Arg( "format", "json" ) );

	var restBlob = "";

	for( var i = 0; i < argList.length; i++ )
	{
		restBlob += argList[i].member + "=" + escape( argList[i].value ) + "&";
	}

	var restRequest = new URL( );		

	restRequest.location = "http://api.flickr.com/services/rest/";
	restRequest.postData = restBlob;

	var tmp =  restRequest.fetch();
	//print(tmp);
	return eval( tmp );
}

// -------------------------------------------------------------------------------------------
function setupTimer() {
   print("setupTimer(" + preferences.timerIntervalPref.value + ")")
    timer.interval = parseInt(preferences.timerIntervalPref.value) * 60;
     // Stopping and restarting the timer resets when it will go off
    timer.ticking = false;
    timer.ticking = true;
}

// -------------------------------------------------------------------------------------------
function mouseEnterPlayBtn() {
	if (timer.ticking) {
		play.src = "Resources/pause-down.png";
	} else {
		play.src = "Resources/play-down.png";
	}
}

// -------------------------------------------------------------------------------------------
function mouseExitPlayBtn() {
	if (timer.ticking) {
		play.src = "Resources/pause.png";
	} else {
		play.src = "Resources/play.png";
	}
}

// -------------------------------------------------------------------------------------------
function pauseLantern() {
	if (timer.ticking) {
		timer.ticking = false;
	} else {
		timer.ticking = true;
	}
	mouseExitPlayBtn();
}

// -------------------------------------------------------------------------------------------
function updateBehavior()	{
	if (!(filesystem.isDirectory(preferences.imageFolder.value))) {
		preferences.imageFolder.value = system.userPicturesFolder;
	}
    flickrPages = [1];
    collectImages();
	// This sets up the next flickr Call to pick a random page, instead of always the first one.
	if (useFlickr) {
		ImageList = [];
		collectImages();
	}	
	showNextImage();
}
// -------------------------------------------------------------------------------------------
// This function pulled from the Yahoo Flickr Widget
function prefChanged( pref )
{
	print(pref," = ",preferences[pref].value);
	if( preferences[pref].value != oldPrefs[pref] )
		return true;
}

function showPhotographer()
{
	print("showPhotographer()");
	openURL("http://flickr.com/photos/" + pickedImage.owner + "/");	
}

function showPhotoOnFlickr()
{
	print("showPhotoOnFlickr()");	
	openURL("http://flickr.com/photos/" + pickedImage.owner + "/" + pickedImage.id + "/");	
}


// -------------------------------------------------------------------------------------------
function infoWindowSetPosition() {
	artist_info.hoffset = screen.availWidth - 352;
	artist_info.voffset = screen.availHeight - 80;
}
// -------------------------------------------------------------------------------------------
// Main
print(runCommand(convert + "-version"));
updateBehavior();

// -------------------------------------------------------------------------------------------
